@@ -43,9 +43,9 @@ class DataSource:
         written to and non-zero when the file is complete. Used to stop
         the iterator without waiting for the timeout.
 
-    cache_dataset: bool (optional - deprecated)
-        Deprecated - Now does nothing - will be removed in future release, not holding the dataset
-        open leads to performance issues.
+    cache_dataset: bool (optional)
+        Hold a reference to the dataset. Can lead to an increase in read performance,
+        but can cause issues with SWMR and VDS.
 
     use_direct_chunk: bool (optional)
         If dataset chunking is aligned to a single frame, and data is blosc
@@ -77,14 +77,16 @@ class DataSource:
         dataset_paths,
         timeout=10,
         finished_dataset=None,
-        cache_datasets=True,
+        cache_datasets=False,
         use_direct_chunk=False,
         interleaved_paths=None,
     ):
         self.h5file = h5file
         self.dataset_paths = dataset_paths
         self.interleaved_paths = interleaved_paths
+        self.max_index = -1
         self.cache = {}
+        self.cache_datasets = cache_datasets
         self.interleaved_cache = {}
         self.kf = KeyFollower(h5file, keypaths, timeout, finished_dataset)
         self.kf.check_datasets()
@@ -115,6 +117,7 @@ class DataSource:
                         self.h5file,
                         self.kf.scan_rank,
                         use_direct_chunk=use_direct_chunk,
+                        cache_datasets=self.cache_datasets,
                     )
 
                     self.interleaved_cache[path].append(fr)
@@ -125,30 +128,46 @@ class DataSource:
     def __next__(self):
 
         current_dataset_index = next(self.kf)
+        force_refresh = False
+        if self.max_index < self.kf.current_max:
+            self.max_index = self.kf.current_max
+            force_refresh = True
 
         output = SliceDict()
 
-        self._add_datasets_to_output(current_dataset_index, output)
-        self._add_interleaved_datasets_to_output(current_dataset_index, output)
+        self._add_datasets_to_output(current_dataset_index, output, force_refresh)
+        self._add_interleaved_datasets_to_output(
+            current_dataset_index, output, force_refresh
+        )
 
         return output
 
-    def _add_datasets_to_output(self, current_dataset_index, output):
+    def _add_datasets_to_output(self, current_dataset_index, output, force_refresh):
         if self.dataset_paths is None:
             return
 
         for path in self.dataset_paths:
-            # If caching requested use from cache
-            # else create
-            fg = self.cache.get(path, FrameReader(path, self.h5file, self.kf.scan_rank))
-            frame, slice_metadata = fg.read_frame(current_dataset_index)
+            fg = self.cache.get(
+                path,
+                FrameReader(
+                    path,
+                    self.h5file,
+                    self.kf.scan_rank,
+                    cache_datasets=self.cache_datasets,
+                ),
+            )
+            frame, slice_metadata = fg.read_frame(
+                current_dataset_index, force_refresh=force_refresh
+            )
             output[path] = frame
             if output.slice_metadata is None:
                 output.slice_metadata = slice_metadata
                 output.maxshape = self.kf.maxshape
                 output.index = current_dataset_index
 
-    def _add_interleaved_datasets_to_output(self, current_dataset_index, output):
+    def _add_interleaved_datasets_to_output(
+        self, current_dataset_index, output, force_refresh
+    ):
         if self.interleaved_paths is None:
             return
 
@@ -157,7 +176,7 @@ class DataSource:
             fr_index = current_dataset_index % (n_frs)
 
             frame, slice_metadata = frs[fr_index].read_frame(
-                current_dataset_index // n_frs
+                current_dataset_index // n_frs, force_refresh=force_refresh
             )
             output[path] = frame
 
@@ -173,6 +192,7 @@ class DataSource:
     def reset(self):
         """Reset the iterator to start again from frame 0"""
         self.kf.reset()
+        self.max_index = -1
 
     def create_dataset(self, data, fh, path):
         scan_max = self.kf.maxshape
@@ -182,7 +202,10 @@ class DataSource:
         return append_data(data, slice_metadata, dataset)
 
     def is_scan_finished(self):
-        return self.kf._prelim_finished_check
+        return self.kf._finish_tag
+
+    def has_timed_out(self):
+        return self.kf.timed_out
 
 
 class SliceDict(dict):
@@ -201,13 +224,13 @@ class FrameReader:
 
     Parameters
     ----------
+    dataset : str
+        The full path to the dataset to extract frames from in the hdf5_File
+
     h5file : h5py.File
         Instance of h5py.File object. Choose the file containing dataset you
         want to extract frames from.
 
-    dataset : str
-        The full path to the dataset to extract frames from in the hdf5_File
-
     scan_rank: int
         The rank of the "non-data-frame" part of the N-dimensional dataset
         for example 1 if the dataset is rank 3 and the frames are images (2D) or
@@ -215,6 +238,15 @@ class FrameReader:
         (i.e. a grid scan). The KeyFollower class has the scan_rank as an attribute
         that can be used here.
 
+    use_direct_chunk: bool (optional)
+        If dataset chunking is aligned to a single frame, and data is blosc
+        compressed, will use direct chunk read and compression outside of h5py
+        for performance.
+
+    cache_datasets: bool (optional)
+        Hold a reference to the dataset. Can lead to an increase in read performance,
+        but can cause issues with SWMR and VDS.
+
     Examples
     --------
 
@@ -225,19 +257,24 @@ class FrameReader:
 
     """
 
-    def __init__(self, dataset, h5file, scan_rank, use_direct_chunk=False):
+    def __init__(
+        self, dataset, h5file, scan_rank, use_direct_chunk=False, cache_datasets=False
+    ):
         self.dataset = dataset
         self.h5file = h5file
         self.scan_rank = scan_rank
         self.use_direct_chunk = use_direct_chunk
-        self.ds = self.h5file[self.dataset]
+        self.ds = None
+        if cache_datasets:
+            self.ds = self.h5file[self.dataset]
 
         if use_direct_chunk:
+            ds = self.ds if self.ds is not None else self.h5file[self.dataset]
             self.use_direct_chunk = False
-            prop_dcid = self.ds.id.get_create_plist()
+            prop_dcid = ds.id.get_create_plist()
             if prop_dcid.get_nfilters() == 1 and prop_dcid.get_filter(0)[0] == 32001:
                 chunk = prop_dcid.get_chunk()
-                shape = self.ds.shape
+                shape = ds.shape
                 frame_rank = len(shape) - scan_rank
                 if shape[-frame_rank:] == chunk[-frame_rank:] and all(
                     [i == 1 for i in chunk[:scan_rank]]
@@ -246,7 +283,7 @@ class FrameReader:
                         self.use_direct_chunk = True
                         self.chunk = chunk
 
-    def read_frame(self, index):
+    def read_frame(self, index, force_refresh=False):
         """Method for using an index from KeyFollower to extract that frame
         from the chosen hdf5 dataset.
 
@@ -256,6 +293,9 @@ class FrameReader:
             Index for the correspondng non-zero unique key generated by an
             instance of KeyFollower.Follower
 
+        force_refresh: bool (optional)
+        Forces refresh to be called on the dataset before the frame is read
+
 
         Examples
         --------
@@ -276,6 +316,9 @@ class FrameReader:
         ds = self.ds if self.ds is not None else self.h5file[self.dataset]
         shape = ds.shape
 
+        if force_refresh and hasattr(ds, "refresh"):
+            ds.refresh()
+
         try:
             # might fail if dataset is cached
             pos = self.get_pos(index, shape)
